
      /*
       *  cpuid.h
       *
       *  Copyright (c) 2007 by Kuscsik Peter <kuscsikp@gmail.com>
       *
       *  This program is free software; you can redistribute it and/or modify
       *  it under the terms of the GNU General Public License as published by
       *  the Free Software Foundation; either version 2 of the License, or
       *  (at your option) any later version.
       *
       *  This program is distributed in the hope that it will be useful,
       *  but WITHOUT ANY WARRANTY; without even the implied warranty of
       *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       *  GNU General Public License for more details.
       *
       *  You should have received a copy of the GNU General Public License
       *  along with this program; if not, write to the Free Software
       *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
       *  USA.
       */ 



	aurora aur("cpuid.aur");
	int *cpuid_buff = (int *)malloc(1024);

void init_cpuid()
{

	int a,b,c,d,i;
	a=b=c=d=0;
	cpuid(0,a,b,c,d);			//create buffer filled with CPUID results
	cpuid_buff[0]=a;
	cpuid_buff[1]=b;
	cpuid_buff[2]=d;
	cpuid_buff[3]=c;
	
	for (i=1;i<12;i++) {
		a=b=c=d=0;
		cpuid(i,a,b,c,d);
		cpuid_buff[i*4]=a;
		cpuid_buff[i*4+1]=b;
		cpuid_buff[i*4+2]=c;
		cpuid_buff[i*4+3]=d;
	}

	for (i=0;i<=32;i++) {
		a=b=c=d=0;
		cpuid(i+0x80000000,a,b,c,d);
		cpuid_buff[(i+12)*4]=a;
		cpuid_buff[(i+12)*4+1]=b;
		cpuid_buff[(i+12)*4+2]=c;
		cpuid_buff[(i+12)*4+3]=d;
	}


	aur.exec_script((const char *)cpuid_buff);	
}

void putchar_(char c){
	if (c<32) printf("�");
	else printf("%c",c);
}

void dump_cpuid()
{
	int i;
	printf("CPUID level  Hexadecimal                                   ASCII\n");
	for (i=0;i<12;i++) {
		printf("0x%08X : 0x%08X 0x%08X 0x%08X 0x%08X   ",i,cpuid_buff[i*4],cpuid_buff[i*4+1],cpuid_buff[i*4+2],cpuid_buff[i*4+3]);	
		for (int j=0; j<4; j++) {
			putchar_(cpuid_buff[i*4+j]);
			putchar_(cpuid_buff[i*4+j]>>8);
			putchar_(cpuid_buff[i*4+j]>>16);
			putchar_(cpuid_buff[i*4+j]>>24);
			printf(" ");			
			} 
		printf("\n");
		}
	printf("Extended CPUID level\n");

	for (i=12;i<32;i++) {
		printf("0x8%07X : 0x%08X 0x%08X 0x%08X 0x%08X   ",i-12,cpuid_buff[i*4],cpuid_buff[i*4+1],cpuid_buff[i*4+2],cpuid_buff[i*4+3]);	
		for (int j=0; j<4; j++) {
			putchar_(cpuid_buff[i*4+j]);
			putchar_(cpuid_buff[i*4+j]>>8);
			putchar_(cpuid_buff[i*4+j]>>16);
			putchar_(cpuid_buff[i*4+j]>>24);
			printf(" ");			
			} 
		printf("\n");
		}
}

#define WIDTH 38

void show_data_long(const char *string)
{
	au_data* data = aur.find_data(string);
	if (data==NULL) {
		return;
		printf("  %-45s : [NOT AVAILABLE]\n",string); 
		}
	if (data->type<=DT_DWORD)
		printf("  %-45s : %d\n",data->comment,data->get_data_int()); 
	else {	
		int ti = 0;
		printf("  %-45s : %.20s\n",data->comment,data->get_data_string()); 
//		printf("  %-20s : %.20s\n",data->comment,temp_string); 
		}
}


void show_data(const char *string)
{
	au_data* data = aur.find_data(string);
	if (data==NULL) {
		return;
		printf("  %-20s : [NOT AVAILABLE]\n",string); 
		}
	if (data->type<=DT_DWORD)
		printf("  %-20s : %d\n",data->comment,data->get_data_int()); 
	else {	
		int ti = 0;
		char *temp_string = data->get_data_string();
		while (*temp_string==' ') temp_string++;	
		printf("  %-20s : ",data->comment);		
		while (*temp_string!='\0'){
			printf("%c",*temp_string);
			temp_string++;
			ti++;
			if ((ti>WIDTH) && (*temp_string==' ')) {
				printf("\n                        ");
				ti=0;

				}
			}
		printf("\n");
//		printf("  %-20s : %.20s\n",data->comment,temp_string); 
		}
}

void show_data_pair(const char *string1,const char *string2)
{
	au_data* data1 = aur.find_data(string1);
	au_data* data2 = aur.find_data(string2);
	if (data1==NULL) {
		show_data(string2);
		return;
		}
	if (data2==NULL) {
		show_data(string1);
		return;
		}

	if (data1->type<=DT_DWORD)
		printf("  %-20s : %-10d",data1->comment,data1->get_data_int()); 
	else
		printf("  %-20s : %-10s",data1->comment,data1->get_data_string()); 

	if (data2->type<=DT_DWORD)
		printf("  %-20s : %-10d\n",data2->comment,data2->get_data_int()); 
	else
		printf("  %-20s : %-10s\n",data2->comment,data2->get_data_string()); 

}

static long time_temp;



long GetCPUTicks(){
	return rdtsc();
}

#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

double calc_proc_speed ()
{
	//calculate CPU speed (in MHz)
	#ifdef __WIN32__
	time_t timeStop, timeStart;
    	timeStart = time (NULL);	
	#else
	unsigned int timeStop, timeStart;
	#endif
	unsigned long t1,t2;
	#ifdef __WIN32__
	#else	
	tms tm;
	#endif
	double speed_mhz=0; 	
  
		#ifdef __WIN32__
	    	timeStart = time (NULL);	
		#else
	    	timeStart = (unsigned int)times(&tm)*10;
		#endif	

		for (;;){
			#ifdef __WIN32__
			timeStop = time (NULL);
			#else
  			timeStop = (unsigned int)times(&tm)*10;
			#endif  			
			if ((unsigned int)((unsigned int)timeStop - (unsigned int)timeStart) > 0){
    				t1 = GetCPUTicks();
    				break;			
			}
		}

		for (;;){
 			#ifdef __WIN32__
			timeStop = time (NULL);			
			if ((int)(timeStop - timeStart) > 1){
    				t2 = GetCPUTicks();
    				break;
			#else
  			timeStop = (unsigned int)times(&tm)*10;
			if ((unsigned int)((unsigned int)timeStop - (unsigned int)timeStart) > 1000){
    				t2 = GetCPUTicks();
    				break;
			#endif
  			}
		}
		speed_mhz = (t2-t1) / (1000.0 * 1000.0);
		//printf("  CPU Speed            : %gMhz\n", speed_mhz / (i+1));

	return speed_mhz;
}

#define INTEL_CPU 1
#define AMD_CPU 2

void correct_fm (int* c_family, int* c_model)
{
	int family = aur.find_data("FAMILY")->get_data_int();
	int model = aur.find_data("MODEL")->get_data_int();
	int ext_family = aur.find_data("EXT_FAMILY")->get_data_int();
	int ext_model = aur.find_data("EXT_MODEL")->get_data_int();
	*c_family=family;
	*c_model=model;
	if (aur.find_data("CPU_VEN")->get_data_int()==INTEL_CPU) {
		if (family!=0x0f) *c_family+=ext_family;
		if ((family == 0x06) || (family == 0x0F))
			*c_model = (ext_model << 4) + model;			

	}
	if (aur.find_data("CPU_VEN")->get_data_int()==AMD_CPU) {
		*c_family += ext_family << 4;
		*c_model += ext_model << 4;			

	}

}


void show_cpuid_short ()
{
	int c_f, c_m;
	correct_fm(&c_f,&c_m);
 	printf("%s,",aur.find_data("VENDOR")->get_data_string());
 	printf("%d,",c_f);
 	printf("%d,",c_m);
 	printf("%d\n",aur.find_data("STEPPING")->get_data_int());
}

void show_xml_data(const char *string)
{
	au_data* data = aur.find_data(string);
	if (data==NULL) {
		return;
		}
	if (data->type<=DT_DWORD)

		printf("                <item name=\"%s\" type=\"INT\" comment=\"%s\" value=\"%d\" ></item>\n",string,data->comment,data->get_data_int()); 
	else {	
		int ti = 0;
		char *temp_string = data->get_data_string();
		while (*temp_string==' ') temp_string++;	
		printf("                <item name=\"%s\" type=\"STRING\" comment=\"%s\" value=\"%s\" ></item>\n",string, data->comment,temp_string); 
//		printf("  %-20s : %.20s\n",data->comment,temp_string); 
		}
}


void show_xml_cpuid()
{
	printf("<procbench>\n");
	printf("        <section name=\"CPU basic information\">\n");
		show_xml_data("BRAND_NAME");		
		printf("\t\t<item name=\"FREQUENCY\" type=\"STRING\" comment=\"Processor frequency\" value=\"%gMhz\" ></item>\n",calc_proc_speed());

		show_xml_data("VENDOR");
		show_xml_data("MAX_LEVEL");
		show_xml_data("MAX_EXT_LEVEL");	
		show_xml_data("FAMILY");
		show_xml_data("EXT_FAMILY");
		show_xml_data("MODE");
		show_xml_data("EXT_MODEL");
		show_xml_data("STEPPING");
		show_xml_data("P_TYPE");
		int c_f, c_m;
		correct_fm(&c_f,&c_m);
		printf("                <item name=\"CMODEL\" type=\"STRING\" comment=\"Correct model\" value=\"%d\"></item>>\n", c_m);
		printf("                <item name=\"CFAMILY\" type=\"STRING\" comment=\"Correct family\" value=\"%d\"></item>\n", c_f);  

	printf("        </section>\n");
	printf("        <section name=\"CPU features\">\n");
		show_xml_data("FEATURES");
		show_xml_data("EXT_FEATURES");
		show_xml_data("MISC_FEATURES");
		show_xml_data("MISC_EXT_FEATURES");
		show_xml_data("APMI");
		show_xml_data("SVM_FEAT");
		show_xml_data("LinAddrSize");
		show_xml_data("PhysAddrSize");

	printf("        </section>\n");
	printf("        <section name=\"CPU TLB infromation\">\n");
		show_xml_data("L1ITlb2and4MSize");
		show_xml_data("L1ITlb2and4MAssoc");
		show_xml_data("L0DTlb2and4MSize");
		show_xml_data("L0DTlb2and4MAssoc");
		show_xml_data("L1DTlb2and4MSize");
		show_xml_data("L1DTlb2and4MAssoc");

		show_xml_data("L1ITlb4KSize");
		show_xml_data("L1ITlb4KAssoc");
		show_xml_data("L0DTlb4KSize");
		show_xml_data("L0DTlb4KAssoc");
		show_xml_data("L1DTlb4KSize");
		show_xml_data("L1DTlb4KAssoc");
	printf("        </section>\n");

	printf("        <section name=\"CPU cache information\">\n");
		show_xml_data("L1IcSize");
		show_xml_data("L1IcAssoc");
		show_xml_data("L1IcLinesPerTag");
		show_xml_data("L1IcLineSize");

		show_xml_data("L1DcSize");
		show_xml_data("L1DcAssoc");
		show_xml_data("L1DcLinesPerTag");
		show_xml_data("L1DcLineSize");

		show_xml_data("L2Size");
		show_xml_data("L2Assoc");
		show_xml_data("L2LinesPerTag");
		show_xml_data("L2LineSize");

		if (aur.find_data("L3Size")!=NULL)				//L3 available
			if (aur.find_data("L3Size")->get_data_int()!=0) {
				show_xml_data("L3Size");
				show_xml_data("L3Assoc");
				show_xml_data("L3LinesPerTag");
				show_xml_data("L3LineSize");
				}
		show_xml_data("Prefetching");
		show_xml_data("Tracecache");
		show_xml_data("TracecacheAssoc");
	printf("        </section>\n");

	printf("</procbench>\n");	
}
void show_cpuid()
{
	printf("\n------=[Base Information]=--------------------------------------------\n");
	show_data("BRAND_NAME");		
//	printf("%s\n",aur.find_data("BRAND_NAME")->get_data_string());
	show_data("VENDOR");
	printf("  CPU Speed            : %gMhz\n", calc_proc_speed());
	printf("\n");
	show_data("MAX_LEVEL");
	show_data("MAX_EXT_LEVEL");	
	show_data_pair("FAMILY","EXT_FAMILY");
	show_data_pair("MODEL","EXT_MODEL");
	show_data_pair("STEPPING","P_TYPE");
	int c_f, c_m;
	correct_fm(&c_f,&c_m);
	printf("  Correct family       : %-10d  Correct Model        : %d",c_f,c_m);

	//printf("  Name             : %s \n",aur.find_data("BRAND_NAME")->get_data_string());
	//printf("  Vendor           : %s \n",aur.find_data("VENDOR")->get_data_string());

	printf("\n");
	
/*	show_data("CORES_P_PACK");
	show_data("LOGP_COUNT");
	show_data("LOG_PROC_SHARE_CACHE");
*/	
	printf("\n");
	printf("------=[Features]=----------------------------------------------------\n");
	show_data("FEATURES");
	show_data("EXT_FEATURES");
	show_data("MISC_FEATURES");
	show_data("MISC_EXT_FEATURES");
	show_data("APMI");
	show_data("SVM_FEAT");
	show_data_long("LinAddrSize");
	show_data_long("PhysAddrSize");
	printf("\n");
	printf("------=[TLB information]=---------------------------------------------\n");
	show_data_long("L1ITlb2and4MSize");
	show_data_long("L1ITlb2and4MAssoc");
	show_data_long("L0DTlb2and4MSize");
	show_data_long("L0DTlb2and4MAssoc");
	show_data_long("L1DTlb2and4MSize");
	show_data_long("L1DTlb2and4MAssoc");

	show_data_long("L1ITlb4KSize");
	show_data_long("L1ITlb4KAssoc");
	show_data_long("L0DTlb4KSize");
	show_data_long("L0DTlb4KAssoc");
	show_data_long("L1DTlb4KSize");
	show_data_long("L1DTlb4KAssoc");

/*	show_data_long("L2ITlb2and4MSize");
	show_data_long("L2ITlb2and4MAssoc");
	show_data_long("L2DTlb2and4MSize");
	show_data_long("L2DTlb2and4MAssoc");

	show_data_long("L2ITlb4KSize");
	show_data_long("L2ITlb4KMAssoc");
	show_data_long("L2DTlb4KSize");
	show_data_long("L2DTlb4KAssoc"); */
	
	printf("\n");
	printf("------=[Cache information]=-------------------------------------------\n");
	show_data_long("L1IcSize");
	show_data_long("L1IcAssoc");
	show_data_long("L1IcLinesPerTag");
	show_data_long("L1IcLineSize");

	show_data_long("L1DcSize");
	show_data_long("L1DcAssoc");
	show_data_long("L1DcLinesPerTag");
	show_data_long("L1DcLineSize");

	show_data_long("L2Size");
	show_data_long("L2Assoc");
	show_data_long("L2LinesPerTag");
	show_data_long("L2LineSize");

	if (aur.find_data("L3Size")!=NULL)				//L3 available
		if (aur.find_data("L3Size")->get_data_int()!=0) {
			show_data_long("L3Size");
			show_data_long("L3Assoc");
			show_data_long("L3LinesPerTag");
			show_data_long("L3LineSize");
			}
	show_data_long("Prefetching");
	show_data_long("Tracecache");
	show_data_long("TracecacheAssoc");
}
