
      /*
       *  main.cpp
       *
       *  Copyright (c) 2008 by Kuscsik Peter <kuscsikp@gmail.com>
       *
       *  This program is free software; you can redistributeprocs_online it and/or modify
       *  it under the terms of the GNU General Public License as published by
       *  the Free Software Foundation; either version 2 of the License, or
       *  (at your option) any later version.
       *
       *  This program is distributed in the hope that it will be useful,
       *  but WITHOUT ANY WARRANTY; without even the implied warranty of
       *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       *  GNU General Public License for more details.
       *
       *  You should have received a copy of the GNU General Public License
       *  along with this program; if not, write to the Free Software
       *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
       *  USA.
       */ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#ifdef __WIN32__
#else
#include "sys/time.h"
#include <sys/times.h>
#endif

#include "bench/ctest.h"
#include "cross.h"
#include "aurora.h"
#include "cpuid.h"
#include "mem_test.h"

const char *host_detect_local_cpu ();
const char *host_detect_local_cpu_old ();
void *host_detect_multi_cpu ();
int *PBBench(int arg);


void print_help()
{
	printf("Usage:	procbench [options]\n");
	printf("List information about (current) x86 processor\n");
	printf("CPU information Options:\n"); 
	printf("	-c	show CPUID information\n");
	printf("	-x	generate XML output from CPUID\n");
	printf("	-d	dump CPUID\n");
	printf("	-f	display best optimisation flags for GCC\n");

	printf("Benchmark Options:\n"); 
	printf("	-l	list CPU benchmarks\n");
	printf("	-r	run all benchmarks\n");
	printf("	-m	memory transfer speed test\n");
	printf("	-i	instruction latency meter (experimental)\n");
	
	printf("Miscellaneous Options:\n"); 
	printf("	-h	display this help and exit\n");
	printf("	-v	display version number\n");
}


int main(int argc, char *argv[])
{
int c;
cmem mem;

while ((c = getopt(argc, argv, ":idmcxfhvlr")) != -1) 
	{
	switch(c){
		case 'c':	/*Instruction latency meter*/
			init_cpuid();
			host_detect_multi_cpu();	
			show_cpuid();
			return 0;
		break;

		case 'f':
			printf("-march=%s -O2 -pipe -fomit-frame-pointer\n", host_detect_local_cpu_old());	
			return 0;
		break;
	

		case 'x':	/*Generate XML output*/
			init_cpuid();
			show_xml_cpuid();
			return 0;
		break;

		case 'd':	/*Dump CPUID*/
			init_cpuid();		 
			dump_cpuid();
			return 0;
		break;
		case 'm':
			mem.mem_speed_test();
			return 0;
		break;

		case 'i':
			mem.instr_test();
			return 0;
		break;

		case 'l':
			PBBench(LIST_BENCHMARKS);
			return 0;
		break;

		case 'r':
			PBBench(RUN_ALL_BENCHMARKS);
			return 0;
		break;

		case 'v':
			printf("Procbench version 0.8.3a\n");
			return 0;
		break;

		case 'h':
			print_help();
			return 0;
		break;

		case '?':	/* help */
			print_help();
			return 0;
		break;

		return 0;
	
		break;
	    }//case
	}
print_help();
}
